C$Procedure      EKLBL ( Generate an MGSO or PDS EK label file )
 
      SUBROUTINE EKLBL ( LBLTYP, EKFNM, KWTBLE, KWPTR, KWVALS )
 
C$ Abstract
C
C     Collect label information from a specified EK file and generate
C     labels values for creating an EK label.
C
C$ Copyright
C
C     Copyright (1997), California Institute of Technology.
C     U.S. Government sponsorship acknowledged.
C
C$ Required_Reading
C
C     None
C
C$ Keywords
C
C     None.
C
C$ Declarations
 
      IMPLICIT NONE
      INTEGER               LBCELL
      PARAMETER           ( LBCELL = -5 )
 
      CHARACTER*(*)         LBLTYP
      CHARACTER*(*)         EKFNM
      CHARACTER*(*)         KWTBLE( LBCELL:*)
      INTEGER               KWPTR ( LBCELL:*)
      CHARACTER*(*)         KWVALS( LBCELL:*)
 
C$ Brief_I/O
C
C     Variable  I/O  Description
C     --------  ---  --------------------------------------------------
C      LBLTYP    I    The type of label file to produce: MGSO or PDS.
C      EKFNM     I    The name of the EK file to use.
C      KWTBLE   I/O   The names portion of a symbol table
C      KWPTR    I/O   The pointers portion of a symbol table
C      KWVALS   I/O   The values portion of a symbol table
C
C$ Detailed_Input
C
C     LBLTYP   The type of label file to be produced. The allowed
C              types are: 'MGSO' or 'PDS'.
C
C     EKFNM    The name of a binary EK file to use when generating
C              values for keywords in the label file. This file must
C              already exist.
C
C     KWTBLE   The symbol table of keyword value lists that will
C     KWPTR    later be converted into an label file for the
C     KWVALS   input EK file.  On input, this symbol table contains
C              the values that were supplied in the MAKLABEL
C              template file.
C
C
C$ Detailed_Output
C
C     KWTBLE   The symbol table of keyword value lists that will
C     KWPTR    later be converted into an label file for the
C     KWVALS   input EK file.  On output, this symbol table contains
C              the values that were supplied in the MAKLABEL
C              template file as well as all values that are
C              appropriate for an EK label.
C
C$ Parameters
C
C     None.
C
C$ Exceptions
C
C     1) If the label type is not one of 'MGSO' or 'PDS' then the error
C        MAKLABEL(BADLABELTYPE) will be signalled.
C
C     2) If the EK SPICE kernel file does not exist, the error
C        MAKLABEL(FILEDOESNOTEXIST) will be signalled.
C
C     3) If the type of the SPICE kernel file cannot be
C        determined, the error MAKLABEL(UNKNOWNFILETYPE) will be
C        signalled.
C
C     4) If the SPICE binary kernel file is not an EK file, the error
C        MAKLABEL(NOTANEKFILE) will be signalled.
C
C$ Files
C
C     None.
C
C$ Particulars
C
C     This subroutine scans an EK file collecting
C     information used to generate a set of labels.
C
C$ Examples
C
C     None.
C
C$ Restrictions
C
C     This subroutine is to be used only with the MAKLABEL program.
C
C$ Literature_References
C
C     None.
C
C$ Author_and_Institution
C
C     B.V. Semenov       (JPL)
C     W.L. Taber         (JPL)
C
C$ Version
C
C-    Beta Version 1.1.0, 9-JUL-1998 (BVS)
C
C        Keyword INSTRUMENT_ID was changed to NAIF_INSTRUMENT_ID.
C
C-    Beta Version 1.0.0, 13-JAN-1997 (WLT)
C
C        Fixed the assignment for KERNEL_TYPE_ID.
C
C-&
 
C$ Index_Entries
C
C      create a NAIF label file from an EK file
C
C-&
 
C
C     SPICELIB functions
C
      INTEGER               RTRIM
      INTEGER               CPOSR
 
      LOGICAL               EXISTS
      LOGICAL               FAILED
C
C     Other functions
C
 
C
C     Local parameters
C
C     Template for the CDB data set ID value.
C
      CHARACTER*(*)         MGSODI
      PARAMETER           ( MGSODI = 'SPICE_EK_FILE' )
 
C
C     Maximum length of an error message.
C
      INTEGER               ERRLEN
      PARAMETER           ( ERRLEN = 132 )
C
C     Maximum length of keyword-value labels.
C
      INTEGER               LBLLEN
      PARAMETER           ( LBLLEN = 132 )
C
C     Maximum length of line in a spice text kernel.
C
      INTEGER               LINLEN
      PARAMETER           ( LINLEN = 255 )
C
C     Maximum length of kernel variable name in a spice text kernel.
C
      INTEGER               KVNLEN
      PARAMETER           ( KVNLEN = 80 )
C
C     Position of the equal sign in a label.
C
      INTEGER               EQUPOS
      PARAMETER           ( EQUPOS = 30 )
C
C     Maximum number of body names that can be stored for a particular
C     file.
C
      INTEGER               MXCRFT
      PARAMETER           ( MXCRFT = 100 )
C
C     Maximum length of spacecraft name.
C
      INTEGER               SNMLEN
      PARAMETER           ( SNMLEN = 80 )
C
C     Length of a file architecture.
C
      INTEGER               ALEN
      PARAMETER           ( ALEN = 3 )
C
C     Length of a file type.
C
      INTEGER               TYPLEN
      PARAMETER           ( TYPLEN = 4 )
 
C
C     Size of a "word"
C
      INTEGER               WDSIZE
      PARAMETER           ( WDSIZE = 32 )
C
C     Maximum length of a time string.
C
      INTEGER               TIMLEN
      PARAMETER           ( TIMLEN = 80 )
 
 
C
C     Local variables
C
      CHARACTER*(ALEN)      ARCH
      CHARACTER*(TIMLEN)    BEGISO
      CHARACTER*(TIMLEN)    ENDISO
      CHARACTER*(TYPLEN)    TYPE
      CHARACTER*(TYPLEN)    MYLBLT
 
 
      CHARACTER*(LBLLEN)    NOTE
      CHARACTER*(LBLLEN)    PRODID
      CHARACTER*(LBLLEN)    VALUE
 
      DOUBLE PRECISION      MINTIM
      DOUBLE PRECISION      MAXTIM
 
      INTEGER               DIRLOC
 
      LOGICAL               HAVTIM
      LOGICAL               FOUND
C
C     Standard SPICE error handling.
C
      CALL CHKIN  ( 'EKLBL' )
 
 
C
C     Check to see if the label type we were given is one we recognize.
C     We only recognize 'PDS' and 'MGSO'.
C
      CALL LJUST ( LBLTYP, MYLBLT )
      CALL LJUST ( MYLBLT, MYLBLT )
 
      IF ( ( MYLBLT .NE. 'PDS' ) .AND. ( MYLBLT .NE. 'MGSO' ) ) THEN
 
         CALL SETMSG ( 'The label type ''#'' was not recognized.'
     .   //            ' Only label types ''PDS'' and ''MGSO'''
     .   //            ' are recognized.'                         )
         CALL ERRCH  ( '#', MYLBLT                                )
         CALL SIGERR ( 'MAKLABEL(BADLABELTYPE)'                   )
         CALL CHKOUT( 'EKLBL'                                    )
         RETURN
 
      END IF
C
C     Check to see that the file exists, and that it is a SPICE file,
C     and hence one we know how to process.
C
      IF ( .NOT. EXISTS ( EKFNM ) ) THEN
 
         CALL SETMSG ( 'The EK file ''#'' does not exist.'    )
         CALL ERRCH  ( '#', EKFNM                              )
         CALL SIGERR ( 'MAKLABEL(FILEDOESNOTEXIST)'             )
         CALL CHKOUT( 'EKLBL'                                  )
         RETURN
 
      END IF
 
      CALL GETFAT ( EKFNM, ARCH, TYPE )
 
      IF ( FAILED() ) THEN
         CALL CHKOUT( 'EKLBL' )
         RETURN
      END IF
 
      IF (  ( ARCH .EQ. '?' ) .OR. ( TYPE .EQ. '?' ) ) THEN
C
C        We didn't have a file that we know how to deal with. So,
C        we toss our hands in the air and signal an error.
C
         CALL SETMSG ( 'The architecture or type of the file'
     .   //            ' ''#'' was not recognized.'            )
         CALL ERRCH  ( '#', EKFNM                             )
         CALL SIGERR ( 'MAKLABEL(UNKNOWNFILETYPE)'             )
         CALL CHKOUT ( 'EKLBL'                                )
         RETURN
      END IF
 
      IF ( TYPE .NE. 'EK' ) THEN
 
         CALL SETMSG ( 'The file ''#'' has a file type, ''#'','
     .   //            ' and this subroutine only works with'
     .   //            ' EK files.'                             )
         CALL ERRCH  ( '#', EKFNM                                )
         CALL ERRCH  ( '#', TYPE                                  )
         CALL SIGERR ( 'MAKLABEL(NOTANEKFILE)'                  )
         CALL CHKOUT( 'EKLBL'                                    )
         RETURN
 
      END IF
 
C
C     At this point, we know that:
C
C        1) We have a SPICE file available for generating labels.
C        2) We know that the label file that we are to produce does not
C           exist, so we can safely create it.
C
C     So we will try to create the label file for the EK file now.
C
      CALL EKTMBD ( EKFNM, MINTIM, MAXTIM, HAVTIM )
 
      IF ( FAILED() ) THEN
         CALL CHKOUT ( 'EKLBL' )
         RETURN
      END IF
 
      IF ( HAVTIM ) THEN
         CALL ET2ISO_M ( MINTIM, 'C', 0, BEGISO )
         CALL ET2ISO_M ( MAXTIM, 'C', 0, ENDISO )
      ELSE
         BEGISO = 'N/A'
         ENDISO = 'N/A'
      END IF
C
C     We have collected everything we plan to get out of the EK
C     kernel.  Here's the stuff we can fill in without any further
C     processing.
C
      NOTE = 'SPICE binary EK file.'
 
      CALL SYIFSC( 'KERNEL_TYPE_ID',    'EK',    KWTBLE, KWPTR, KWVALS)
      CALL SYIFSC( 'START_TIME',        BEGISO,  KWTBLE, KWPTR, KWVALS)
      CALL SYIFSC( 'STOP_TIME',         ENDISO,  KWTBLE, KWPTR, KWVALS)
      CALL SYIFSC( 'TARGET_NAME',       'N/A',   KWTBLE, KWPTR, KWVALS)
      CALL SYIFSC( 'INSTRUMENT_NAME',   'N/A',   KWTBLE, KWPTR, KWVALS)
      CALL SYIFSC( 'NAIF_INSTRUMENT_ID','N/A',   KWTBLE, KWPTR, KWVALS)
      CALL SYIFSC( 'SOURCE_PRODUCT_ID', 'UNK',   KWTBLE, KWPTR, KWVALS)
      CALL SYIFSC( 'NOTE',              NOTE,    KWTBLE, KWPTR, KWVALS)
      CALL SYIFSC( 'SPACECRAFT_NAME',   'N/A',   KWTBLE, KWPTR, KWVALS)
 
      CALL SYIFSC( 'SPACECRAFT_CLOCK_START_COUNT', 'N/A', KWTBLE,
     .                                                    KWPTR,
     .                                                    KWVALS )
      CALL SYIFSC( 'SPACECRAFT_CLOCK_STOP_COUNT',  'N/A', KWTBLE,
     .                                                    KWPTR,
     .                                                    KWVALS )
      CALL SYIFSC( 'PLATFORM_OR_MOUNTING_NAME',    'N/A', KWTBLE,
     .                                                    KWPTR,
     .                                                    KWVALS )
 
C
C     The remaining pieces require a small amount of processing.
C
C     Keyword: PRODUCT_ID
C     ===================
C
C     Set the product ID. This is the filename with all leading
C     directory stuff removed.
C
C     Remove any leading path or directory elements.
C
      DIRLOC = CPOSR ( EKFNM, '/]', RTRIM(EKFNM) )
 
      IF ( DIRLOC .EQ. 0 ) THEN
         PRODID = EKFNM
      ELSE
         PRODID = EKFNM(DIRLOC+1:)
      END IF
 
      CALL SYIFSC( 'PRODUCT_ID',        PRODID, KWTBLE, KWPTR, KWVALS )
 
C
C     Keyword: DATA_SET_ID
C     ====================
C
C     Look up the current value for the DATA_SET_ID.  This will
C     be overridden by the SYPOPC call below if anything at all
C     is present.
C
      VALUE = MGSODI
 
      CALL SYPOPC( 'DATA_SET_ID', KWTBLE, KWPTR, KWVALS,  VALUE, FOUND )
 
      IF ( MYLBLT .EQ. 'PDS' .AND. .NOT. FOUND ) THEN
C
C        If we didn't get a value back from the call above
C        we are totally out of luck.
C
         CALL SETMSG ( 'There was no template value supplied '
     .   //            'for the DATA_SET_ID in the TEMPLATE '
     .   //            'file.  Until that information is '
     .   //            'supplied the label file can not be '
     .   //            'created. ' )
         CALL SIGERR ( 'MAKLABEL(NODATASETID)' )
         CALL CHKOUT ( 'EKLBL' )
         RETURN
 
      END IF
C
C     A blank value for the DATA_SET_ID is not acceptable.
C
      IF ( VALUE .EQ. ' ' ) THEN
         VALUE = MGSODI
      END IF
C
C     If the current value is just a template, fill in the marker
C     (this has no effect if there isn't a marker present).
C
      CALL REPMC  ( VALUE, '#',    TYPE,  VALUE )
      CALL SYSETC ('DATA_SET_ID',         VALUE, KWTBLE, KWPTR,KWVALS )
 
C
C     That's all folks!!!
C
      CALL CHKOUT ( 'EKLBL' )
      RETURN
      END
